﻿using Microsoft.Data.SqlClient;
using System;
using System.Data;
using System.Text;

namespace MySecurityProvider
{
	public sealed class Database
	{
		public static string ConnectionString;


		public static void SetConnectionString(string cnStr)
		{
			ConnectionString = cnStr;
		}

		static SqlConnection GetDatabaseConnection()
		{
			SqlConnection cn = new SqlConnection();
			try
			{
				cn.ConnectionString = ConnectionString;
				cn.Open();
				return cn;
			}
			catch (Exception e)
			{
				WriteLogS("GetDatabaseConnection", e.ToString());
				return null;
			}
		}

		public static UserInfo GetUserInfo(string userName, string password)
		{
			try
			{
				using (SqlConnection cn = GetDatabaseConnection())
				{
					if (null == cn) return null;
					using (var cmd = cn.CreateCommand())
					{
						StringBuilder sb = new StringBuilder();
						sb.AppendLine("SELECT * ");
						sb.AppendLine("FROM UserInfo ");
						sb.AppendLine("WHERE UserName=@userName");
						sb.AppendLine(" AND UserPassword=@password ");
						cmd.CommandText = sb.ToString();
						cmd.Parameters.AddWithValue("@userName", userName);
						cmd.Parameters.AddWithValue("@password", password);
						using (var adp = new SqlDataAdapter(cmd))
						{
							DataSet ds = new DataSet();
							adp.Fill(ds);
							if (ds.Tables[0].Rows.Count == 0)
							{
								WriteLogS("GetUserInfo", "No matched username and password.");
								return null;
							}
							DataRow r = ds.Tables[0].Rows[0];

							// create instance
							UserInfo u = new UserInfo();
							u.UserName = (string)r["UserName"];
							u.RoleNames = (string)r["Roles"];
							u.Title = r.IsNull("Title") ? "" : (string)r["Title"];
							u.DepNames = r.IsNull("DepNames") ? "" : (string)r["DepNames"];

							WriteLogS("GetUserInfo", "Matched username and password was found.");

							return u;
						}
					}
				}
			}
			catch (Exception e)
			{
				WriteLogS("GetUserInfo", e.ToString());
				return null;
			}
		}

		private static UserInfo GetUserInfoByName(string userName)
		{
			try
			{
				using (SqlConnection cn = GetDatabaseConnection())
				{
					if (null == cn) return null;
					using (var cmd = cn.CreateCommand())
					{
						StringBuilder sb = new StringBuilder();
						sb.AppendLine("SELECT * ");
						sb.AppendLine("FROM UserInfo ");
						sb.AppendLine("WHERE UserName=@userName");
						//sb.AppendLine(" AND UserPassword=@password ");
						cmd.CommandText = sb.ToString();
						cmd.Parameters.AddWithValue("@userName", userName);
						//cmd.Parameters.AddWithValue("@password", password);
						using (var adp = new SqlDataAdapter(cmd))
						{
							DataSet ds = new DataSet();
							adp.Fill(ds);
							if (ds.Tables[0].Rows.Count == 0) return null;
							DataRow r = ds.Tables[0].Rows[0];

							// create instance
							UserInfo u = new UserInfo();
							u.UserName = (string)r["UserName"];
							u.RoleNames = (string)r["Roles"];
							u.Title = r.IsNull("Title") ? "" : (string)r["Title"];
							u.DepNames = r.IsNull("DepNames") ? "" : (string)r["DepNames"];
							return u;
						}
					}
				}
			}
			catch (Exception e)
			{
				WriteLogS("GetUserInfo", e.ToString());
				return null;
			}
		}

		internal static UserInfo GetUserInfoFromToken(string token)
		{
			if (string.IsNullOrWhiteSpace(token)) return null;
			string[] tokenValues = GetTokenValues(token);
			string userName = tokenValues[0];
			UserInfo userInfo = GetUserInfoByName(userName);
			return userInfo;
		}


		private static string[] GetTokenValues(string token)
		{
			if (string.IsNullOrWhiteSpace(token)) return null;
			var tokenBytes = Convert.FromBase64CharArray(token.ToCharArray(), 0, token.Length);
			token = Encoding.UTF8.GetString(tokenBytes);
			string[] tokenValues = token.Split(Constants.TokenDelimiter[0]);
			return tokenValues;
		}

		public static void WriteLogS(string location, string logInfo)
		{
			try
			{
				string time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
				string s = time + " " + location + " " + logInfo + "\r\n";
				System.IO.File.AppendAllText("C:\\!Log\\Log_SecProv.txt", s);
			}
			catch
			{
				// Ignore errors
			}
		}
	}
}
